/**

Copyright (c) Scott Gasch

Module Name:

    chess.h

Abstract:

Author:

    Scott Gasch (scott.gasch@gmail.com) 7 Apr 2004

Revision History:

    $Id: chess.h 354 2008-06-30 05:10:08Z scott $

**/

#ifndef CHESS_
#define CHESS_

#include "compiler.h"


#if defined(__GNUC__) && !defined(__MINGW32__)

static inline int32_t InterlockedIncrement(volatile int32_t * const lpAddend) {
return __sync_add_and_fetch(lpAddend, 1);
}


static inline int8_t InterlockedCompareExchange(volatile int8_t * const Destination, const int8_t Exchange, const int8_t Comperand) {
 
 	return __sync_bool_compare_and_swap(Destination, Exchange, Comperand);
 }


static inline int8_t _InterlockedExchange(volatile int8_t * const Destination, const int8_t Exchange) {

    __sync_synchronize();
	return __sync_lock_test_and_set(&Destination, Exchange);
}

#endif

//
// Datatype wrappers
//
#define MIN_CHAR                   (0x80)
#define MAX_CHAR                   (0x7f)

#define MIN_BYTE                   (0x00)
#define MAX_BYTE                   (0xff)


#define MIN_UCHAR                  (0x00)
#define MAX_UCHAR                  (0xff)

#define CAN_FIT_IN_UCHAR(x)        ((x) <= MAX_UCHAR)

#define MIN_SHORT                  (0x8000)
#define MAX_SHORT                  (0x7fff)


#define MIN_USHORT                 (0x0000)
#define MAX_USHORT                 (0xffff)

#define CAN_FIT_IN_USHORT(x)       ((x) <= MAX_USHORT)

#define MIN_WORD                   (0x0000)
#define MAX_WORD                   (0xffff)


#define MIN_INT                    (0x80000000)
#define MAX_INT                    (0x7fffffff)

#define MIN_DWORD                  (0x00000000)
#define MAX_DWORD                  (0xffffffff)

#define MIN_UINT                   (0x00000000)
#define MAX_UINT                   (0xffffffff)


#define MIN_ULONG                  (0x00000000)
#define MAX_ULONG                  (0xffffffff)


#define MIN_INT64                  (0x8000000000000000)
#define MAX_INT64                  (0x7fffffffffffffff)

#define MIN_UINT64                 (0x0000000000000000)
#define MAX_UINT64                 (0xffffffffffffffff)

#define MIN_BITBOARD               (0x0000000000000000)
#define MAX_BITBOARD               (0xffffffffffffffff)
typedef uint64_t                     BITBOARD;

#define MIN_BITV                   MIN_UINT
#define MAX_BITV                   MAX_UINT
typedef uint32_t                   BITV;

#define MIN_FLAG                   MIN_UINT
#define MAX_FLAG                   MAX_UINT
typedef uint32_t                   FLAG;
#define IS_VALID_FLAG(x)           (((x) == TRUE) || ((x) == FALSE))

#if defined(__LP64__)
typedef uint64_t                   COOR;
#else
typedef uint32_t                   COOR;
#endif

typedef uint32_t                   PIECE;
typedef int32_t                    SCORE;

#define BIT1                              (0x1)
#define BIT2                              (0x2)
#define BIT3                              (0x4)
#define BIT4                              (0x8)
#define BIT5                             (0x10)
#define BIT6                             (0x20)
#define BIT7                             (0x40)
#define BIT8                             (0x80)
#define BIT9                            (0x100)
#define BIT10                           (0x200)
#define BIT11                           (0x400)
#define BIT12                           (0x800)
#define BIT13                          (0x1000)
#define BIT14                          (0x2000)
#define BIT15                          (0x4000)
#define BIT16                          (0x8000)
#define BIT17                         (0x10000)
#define BIT18                         (0x20000)
#define BIT19                         (0x40000)
#define BIT20                         (0x80000)
#define BIT21                        (0x100000)
#define BIT22                        (0x200000)
#define BIT23                        (0x400000)
#define BIT24                        (0x800000)
#define BIT25                       (0x1000000)
#define BIT26                       (0x2000000)
#define BIT27                       (0x4000000)
#define BIT28                       (0x8000000)
#define BIT29                      (0x10000000)
#define BIT30                      (0x20000000)
#define BIT31                      (0x40000000)
#define BIT32                      (0x80000000)

typedef struct _DLIST_ENTRY
{
    struct _DLIST_ENTRY *pFlink;
    struct _DLIST_ENTRY *pBlink;
} DLIST_ENTRY;

//
// Constants
//
#define YES                        (1)
#define NO                         (0)
#ifndef TRUE
#define TRUE                       (YES)
#endif
#ifndef FALSE
#define FALSE                      (NO)
#endif

//
// Calculate the length of an array (i.e. number of elements)
//
#define ARRAY_LENGTH(x)            (sizeof(x) / sizeof((x)[0]))
#define COMMAND(x) \
    void (x)(int8_t *szInput, uint32_t argc, int8_t *argv[], POSITION *pos)
#define MB (1024 * 1024)

//
// Calculate the address of the base of the structure given its type, and an
// address of a field within the structure.
//
#ifndef OFFSET_OF
#define OFFSET_OF(field, type) \
    (uint32_t)(&((type *)0)->field)
#endif
#ifndef CONTAINING_STRUCT
#define CONTAINING_STRUCT(address, type, field) \
    ((type *)((uint8_t *)(address) - (uint8_t *)(OFFSET_OF(field, type))))
#endif

#define WHITE                      (1)
#define BLACK                      (0)
#define BAD_COLOR                  (2)
#define IS_VALID_COLOR(x)          (((x) == WHITE) || ((x) == BLACK))
#define FOREACH_COLOR(x)           for((x) = BLACK; (x) < BAD_COLOR; (x)++)
#define RANDOM_COLOR               (rand() & 1)
#define FLIP(color)                ((color) ^ 1)
#define COLOR_NAME(color)          (((color) == WHITE) ? "white" : "black")

#define MAX_MOVES_PER_PLY          (218)
#define MAX_PLY_PER_SEARCH         (64)
#define MAX_MOVES_PER_GAME         (1024)
#define SMALL_STRING_LEN_CHAR      (256)
#define MEDIUM_STRING_LEN_CHAR     (8192)
#define BIG_STRING_LEN_CHAR        (16384)

#define QUARTER_PLY                16
#define HALF_PLY                   32
#define THREE_QUARTERS_PLY         48
#define ONE_PLY                    64
#define TWO_PLY                    128
#define THREE_PLY                  192
#define FOUR_PLY                   256
#define MAX_DEPTH_PER_SEARCH       (MAX_PLY_PER_SEARCH * ONE_PLY)

#define IS_VALID_DEPTH(x)          (((x) >= 0) && \
                                   ((x) <= MAX_DEPTH_PER_SEARCH) && \
                                   (((x) & 0xfffff00f) == 0))

#define INFINITY                   (MAX_SHORT)
#define INVALID_SCORE              (INFINITY + 1)
#define IS_VALID_SCORE(x)          (((x) >= -INFINITY) && \
                                    ((x) <= +INFINITY))
#define NMATE                      (+INFINITY - 200)
#define MATED_SCORE(ply)           (-INFINITY + (ply))

#define IS_A_POWER_OF_2(x)         (((x) & (x - 1)) == 0)
//
// Program version number
//
#define VERSION                    "1.00"
#define REVISION                   "$Id: chess.h 354 2008-06-30 05:10:08Z scott $\n"

//
// Function decorators
//
#define IN
#define OUT
#define INOUT
#define UNUSED
#define NOTHING

// ----------------------------------------------------------------------
//
// PIECE:
//
// a piece (4 bits) = type (3 bits) + color (1 bit)
//
//  3   1 0
//  . . . .
// |     | |
//  type  C
//
#define KING                       (6)        // 110X
#define QUEEN                      (5)        // 101X
#define ROOK                       (4)        // 100X
#define BISHOP                     (3)        // 011X
#define KNIGHT                     (2)        // 010X
#define PAWN                       (1)        // 001X
#define EMPTY                      (0)        // 000X

// (WHATEVER << 1)
#define BLACK_PAWN                 (2)        // 0010
#define BLACK_KNIGHT               (4)        // 0100
#define BLACK_BISHOP               (6)        // 0110
#define BLACK_ROOK                 (8)        // 1000
#define BLACK_QUEEN                (10)       // 1010
#define BLACK_KING                 (12)       // 1100

// (WHATEVER << 1) | WHITE
#define WHITE_PAWN                 (3)        // 0011
#define WHITE_KNIGHT               (5)        // 0101
#define WHITE_BISHOP               (7)        // 0111
#define WHITE_ROOK                 (9)        // 1001
#define WHITE_QUEEN                (11)       // 1011
#define WHITE_KING                 (13)       // 1101

#define PIECE_TYPE(p)              ((p) >> 1)
#define PIECE_COLOR(p)             ((p) & WHITE)
#define RANDOM_PIECE               (rand() % 12) + 2;
#define IS_PAWN(p)                 (((p) & 0xE) == BLACK_PAWN)
#define IS_KNIGHT(p)               (((p) & 0xE) == BLACK_KNIGHT)
#define IS_BISHOP(p)               (((p) & 0xE) == BLACK_BISHOP)
#define IS_ROOK(p)                 (((p) & 0xE) == BLACK_ROOK)
#define IS_QUEEN(p)                (((p) & 0xE) == BLACK_QUEEN)
#define IS_KING(p)                 (((p) & 0xE) == BLACK_KING)
#define IS_KNIGHT_OR_KING(p)       (((p) & 0x6) == 0x4)
#define IS_VALID_PIECE(p)          ((PIECE_TYPE((p)) >= PAWN) && \
                                    (PIECE_TYPE((p)) <= KING))
#define IS_WHITE_PIECE(p)          ((p) & WHITE)
#define IS_BLACK_PIECE(p)          (!IS_WHITE(p))
#define OPPOSITE_COLORS(p, q)      (((p) ^ (q)) & WHITE)
#define SAME_COLOR(p,q)            (!OPPOSITE_COLORS(p, q))
#define GET_COLOR(p)               ((p) & WHITE)

// ----------------------------------------------------------------------
//
// a coordinate (8 bits) = rank + file (0x88)
//
//  7     4 3     0
//  . . . . . . . .
// |       |       |
//   rank     file
//     |        |
//     |        +--- 0-7 = file A thru file H
//     |
//     +------------ 0-7 = rank 1 thru rank 8
//
// given a coordinate, C, if C & 0x88 then it is illegal (off board)
//
//
//     0x00 0x01 0x02 0x03 0x04 0x05 0x06 0x07
//    +---------------------------------------+
//  8 | 00 |:01:| 02 |:03:| 04 |:05:| 06 |:07:| 0x00
//  7 |:10:| 11 |:12:| 13 |:14:| 15 |:16:| 17 | 0x10
//  6 | 20 |:21:| 22 |:23:| 24 |:25:| 26 |:27:| 0x20
//  5 |:30:| 31 |:32:| 33 |:34:| 35 |:36:| 37 | 0x30
//  4 | 40 |:41:| 42 |:43:| 44 |:45:| 46 |:47:| 0x40
//  3 |:50:| 51 |:52:| 53 |:54:| 55 |:56:| 57 | 0x50
//  2 | 60 |:61:| 62 |:63:| 64 |:65:| 66 |:67:| 0x60
//  1 |:70:| 71 |:72:| 73 |:74:| 75 |:76:| 77 | 0x70
//    +---------------------------------------+
//       A    B    C    D    E    F    G    H
//
#define A                          (0)
#define B                          (1)
#define C                          (2)
#define D                          (3)
#define E                          (4)
#define F                          (5)
#define G                          (6)
#define H                          (7)

#define A8                         (0x00)
#define B8                         (0x01)
#define C8                         (0x02)
#define D8                         (0x03)
#define E8                         (0x04)
#define F8                         (0x05)
#define G8                         (0x06)
#define H8                         (0x07)

#define A7                         (0x10)
#define B7                         (0x11)
#define C7                         (0x12)
#define D7                         (0x13)
#define E7                         (0x14)
#define F7                         (0x15)
#define G7                         (0x16)
#define H7                         (0x17)

#define A6                         (0x20)
#define B6                         (0x21)
#define C6                         (0x22)
#define D6                         (0x23)
#define E6                         (0x24)
#define F6                         (0x25)
#define G6                         (0x26)
#define H6                         (0x27)

#define A5                         (0x30)
#define B5                         (0x31)
#define C5                         (0x32)
#define D5                         (0x33)
#define E5                         (0x34)
#define F5                         (0x35)
#define G5                         (0x36)
#define H5                         (0x37)

#define A4                         (0x40)
#define B4                         (0x41)
#define C4                         (0x42)
#define D4                         (0x43)
#define E4                         (0x44)
#define F4                         (0x45)
#define G4                         (0x46)
#define H4                         (0x47)

#define A3                         (0x50)
#define B3                         (0x51)
#define C3                         (0x52)
#define D3                         (0x53)
#define E3                         (0x54)
#define F3                         (0x55)
#define G3                         (0x56)
#define H3                         (0x57)

#define A2                         (0x60)
#define B2                         (0x61)
#define C2                         (0x62)
#define D2                         (0x63)
#define E2                         (0x64)
#define F2                         (0x65)
#define G2                         (0x66)
#define H2                         (0x67)

#define A1                         (0x70)
#define B1                         (0x71)
#define C1                         (0x72)
#define D1                         (0x73)
#define E1                         (0x74)
#define F1                         (0x75)
#define G1                         (0x76)
#define H1                         (0x77)

// TODO: if use this anywhere that matters, consider double loop
// and unrolling the inside loop to prevent the multiple
// continue for invalid squares
#define FOREACH_SQUARE(x)          for((x) = (A8); (x) <= (H1); (x)++)

#define IS_ON_BOARD(c)             (!((c) & 0x88))
#define RANDOM_COOR                (rand() & 0x77)
#define FILE_RANK_TO_COOR(f, r)    (((8 - (r)) << 4) | (f))
#define RANK(c)                    (8 - ((c) >> 4))
#define RANK1(c)                   (((c) & 0xF0) == 0x70)
#define RANK2(c)                   (((c) & 0xF0) == 0x60)
#define RANK3(c)                   (((c) & 0xF0) == 0x50)
#define RANK4(c)                   (((c) & 0xF0) == 0x40)
#define RANK5(c)                   (((c) & 0xF0) == 0x30)
#define RANK6(c)                   (((c) & 0xF0) == 0x20)
#define RANK7(c)                   (((c) & 0xF0) == 0x10)
#define RANK8(c)                   (((c) & 0xF0) == 0x00)
#define MYFILE(c)                    ((c) & 0x0F)
#define FILEA(c)                   (((c) & 0x0F) == 0x00)
#define FILEB(c)                   (((c) & 0x0F) == 0x01)
#define FILEC(c)                   (((c) & 0x0F) == 0x02)
#define FILED(c)                   (((c) & 0x0F) == 0x03)
#define FILEE(c)                   (((c) & 0x0F) == 0x04)
#define FILEF(c)                   (((c) & 0x0F) == 0x05)
#define FILEG(c)                   (((c) & 0x0F) == 0x06)
#define FILEH(c)                   (((c) & 0x0F) == 0x07)
#define IS_WHITE_SQUARE_FR(f, r)   (((f) + (r) - 1) & 1)
#define IS_WHITE_SQUARE_COOR(c)    (IS_WHITE_SQUARE_FR(MYFILE(c), RANK(c)))
#define VALID_EP_SQUARE(c)         (((c) == ILLEGAL_COOR) || ((c) == 0) || \
                                    (RANK3(c)) || \
                                    (RANK6(c)))

// ----------------------------------------------------------------------
//
// a move (4 bytes) = from/to, moved, captured, promoted and special
//
//  31           24 23           16 15   12 11    8 7     4 3     0
//  . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
// |     from      |       to      | moved | capt  | prom  | flags |
//         |               |          |       |       |
//         |               |          |       |       |
//         |               |          |       |       |
//         |               |          +-------+-------+-- type and
//         |               |                              color of
//         |               |                              piece moved
//         |               |                              captured and
//         |               |                              promote targ
//         +---------------+-- board location where
//                             move begins and ends
//
//
// Special flag asserted on special move types:
//
// if (special)
// {
//    if (moved == PAWN)
//    {
//       if (promotion)
//       {
//          move is a promote (w/ or w/o capture)
//       }
//       else if (capture)
//       {
//          move is an en passant catpure
//       }
//       else
//       {
//          move is a double jump
//       }
//    }
//    else if (moved == KING)
//    {
//       move is a castle (int32_t or short)
//    }
//    else
//    {
//       invalid use of special flag
//    }
// }
//
typedef union _MOVE
{
    uint32_t uMove;
    struct
    {
        uint8_t cFrom        : 8;
        uint8_t cTo          : 8;
        uint8_t pMoved       : 4;
        uint8_t pCaptured    : 4;
        uint8_t pPromoted    : 4;
        uint8_t bvFlags      : 4;
    };
} MOVE;

#define IS_SAME_MOVE(a, b) \
    (((a).uMove & 0x0FFFFFFF) == (((b).uMove & 0x0FFFFFFF)))

#define MAKE_MOVE(from,to,piece,cap,prom,flags) \
    (((uint32_t)(from)) | \
     ((uint32_t)(to) << 8) | \
     ((uint32_t)(piece) << 16) | \
     ((uint32_t)(cap) << 20) | \
     ((uint32_t)(prom) << 24) | \
     ((uint32_t)(flags) << 28))

#define MAKE_MOVE_WITH_NO_PROM_OR_FLAGS(from,to,piece,cap) \
    (((uint32_t)(from)) | \
     ((uint32_t)(to) << 8) | \
     ((uint32_t)(piece) << 16) | \
     ((uint32_t)(cap) << 20))

//
// Move flag values:
//
#define MOVE_FLAG_SPECIAL          (1)        // en pass, prom, castle, double
#define IS_SPECIAL_MOVE(mv)         \
    ((mv).uMove & 0x10000000)

#define IS_CASTLE(mv)               \
    (IS_SPECIAL_MOVE(mv) && IS_KING((mv).pMoved))

#ifdef DEBUG
#define IS_PROMOTION(mv)            \
    (IS_SPECIAL_MOVE(mv) && ((mv).pPromoted))
#else
#define IS_PROMOTION(mv)            \
    ((mv).pPromoted)
#endif

#define IS_CAPTURE_OR_PROMOTION(mv) \
    ((mv).uMove & 0x0FF00000)

#define IS_ENPASSANT(mv)            \
    (IS_SPECIAL_MOVE(mv) && (mv.pCaptured) && !IS_PROMOTION(mv))

#define IS_DOUBLE_JUMP(mv)          \
    (IS_SPECIAL_MOVE(mv) && !IS_CAPTURE_OR_PROMOTION(mv))

#define MOVE_FLAG_KILLERMATE       (2)        // killer mate
#define IS_KILLERMATE_MOVE(mv)     ((mv).uMove & 0x20000000)

#define MOVE_FLAG_ESCAPING_CHECK   (4)        // escaping check
#define IS_ESCAPING_CHECK(mv)      ((mv).uMove & 0x40000000)

#define MOVE_FLAG_CHECKING         (8)        // checking move
#define IS_CHECKING_MOVE(mv)       ((mv).uMove & 0x80000000)

// ----------------------------------------------------------------------
//
// Move list
//
#define MAX_MOVE_LIST              (MAX_MOVES_PER_GAME + MAX_PLY_PER_SEARCH)

// ----------------------------------------------------------------------

#pragma pack(1)
typedef union _ATTACK_BITV
{
    uint32_t uWholeThing;
    struct
    {
        union
        {
            uint8_t uSmall;
            struct
            {
                uint8_t uNumAttacks : 3;            // 0..2
                uint8_t uKing : 1;                  // 4..7
                uint8_t uQueen : 1;
                uint8_t uRook : 1;
                uint8_t uMinor : 1;
                uint8_t uPawn : 1;
            } little;
        };

        // --------------------
        union
        {
            uint16_t uBig;
            struct
            {
                uint16_t uKing : 1;                  // 8..23
                uint16_t uQueens : 4;
                uint16_t uRooks : 4;
                uint16_t uMinors : 4;
                uint16_t uPawns : 2;
                uint16_t uUnusedFlag1 : 1;
            } big;
        };

        // -------------------
        union
        {
            uint8_t uXray;
            struct
            {
                uint8_t uNumXrays : 3;
                uint8_t uUnusedFlag2 : 1;
                uint8_t uQueen : 1;                 // 24..28
                uint8_t uRook : 1;
                uint8_t uBishop : 1;
                uint8_t uUnusedFlag3 : 1;
            } xray;
        };
    };
}
ATTACK_BITV;

#define UNSAFE_FOR_MINOR(x) ((uint32_t)((x).uWholeThing) & 0x00000080UL)
#define UNSAFE_FOR_ROOK(x)  ((uint32_t)((x).uWholeThing) & 0x000000C0UL)
#define UNSAFE_FOR_QUEEN(x) ((uint32_t)((x).uWholeThing) & 0x000000E0UL)

#define PAWN_BIT       0x00000080UL
#define MINOR_BIT      0x00000040UL
#define MINOR_XRAY_BIT 0x40000000UL
#define ROOK_BIT       0x00000020UL
#define ROOK_XRAY_BIT  0x20000000UL
#define QUEEN_BIT      0x00000010UL
#define QUEEN_XRAY_BIT 0x10000000UL

#define INVALID_PIECE_INDEX (17)
#define IS_VALID_PIECE_INDEX(x) ((x) < INVALID_PIECE_INDEX)

typedef union _SQUARE
{
    struct
    {
        PIECE pPiece;
        uint32_t uIndex;
    };
    ATTACK_BITV bvAttacks[2];
}
SQUARE;
#pragma pack()

//
// POSITION
//
typedef struct _POSITION
{
    SQUARE rgSquare[128];                  // where the pieces are,
                                           // also, the attack table
    uint64_t u64NonPawnSig;                  // hash signature
    uint64_t u64PawnSig;                     // pawn hash signature
    uint32_t uToMove;                         // whose turn?
    uint32_t uFifty;                          // 50 moves w/o progress = draw
    FLAG fCastled[2];                      // record when sides have castled
    BITV bvCastleInfo;                     // who can castle how?
    COOR cEpSquare;                        // en-passant capture square

    COOR cPawns[2][8];                     // location of pawns on the board
    uint32_t uPawnMaterial[2];                // pawn material of each side
    uint32_t uPawnCount[2];                   // number of pawns for each side

    COOR cNonPawns[2][16];                 // location of pieces on the board
    uint32_t uNonPawnMaterial[2];             // piece material of each side
    uint32_t uNonPawnCount[2][8];             // number of non-pawns / type
                                           // 0 and 1 are the sum,
                                           // 2..6 are per PIECE_TYPE

    uint32_t uWhiteSqBishopCount[2];          // num bishops on white squares
    SCORE iMaterialBalance[2];             // material balance

    // temporary storage space for use in eval
    COOR cTrapped[2];
    uint32_t uArmyScaler[2];
    uint32_t uClosedScaler;
    SCORE iScore[2];
    SCORE iReducedMaterialDownScaler[2];
    SCORE iTempScore;
    uint32_t uMinMobility[2];
    COOR cPiece;
    uint32_t uMinorsAtHome[2];
    BITBOARD bb;
    uint32_t uPiecesPointingAtKing[2];
}
POSITION;

//
// Castling permission bitvector flags.
//
#define CASTLE_WHITE_SHORT                (1)
#define CASTLE_WHITE_LONG                 (2)
#define WHITE_CAN_CASTLE                  \
    (CASTLE_WHITE_SHORT | CASTLE_WHITE_LONG)
#define CASTLE_BLACK_SHORT                (4)
#define CASTLE_BLACK_LONG                 (8)
#define BLACK_CAN_CASTLE                  \
    (CASTLE_BLACK_SHORT | CASTLE_BLACK_LONG)
#define CASTLE_NONE_POSSIBLE              (0)
#define CASTLE_ALL_POSSIBLE               \
    (WHITE_CAN_CASTLE | BLACK_CAN_CASTLE)

// ----------------------------------------------------------------------
//
// Move stack
//
#define MAX_MOVE_STACK             (MAX_PLY_PER_SEARCH * \
                                    MAX_MOVES_PER_PLY)

//
// Main part of the move stack, triads of moves, their values and flags
// that denote where the values came from
//
#define MVF_MOVE_SEARCHED    (1)
#define MVF_EXTEND_MOVE      (2)
#define MVF_REDUCE_MOVE      (4)
#define MVF_PRUNE_SUBTREE    (8)

typedef struct _MOVE_STACK_MOVE_VALUE_FLAGS
{
    SCORE iValue;
    MOVE mv;
    BITV bvFlags;
}
MOVE_STACK_MOVE_VALUE_FLAGS;

typedef struct _KEY_POINTER
{
    int32_t iPointer;
    uint32_t uKey;
}
KEY_POINTER;

typedef union _GENERATOR_FLAGS 
{
    struct {
        uint8_t uMoveCount;
        uint8_t uKingMoveCount;


        uint8_t uCheckingPieces;
        uint8_t uUnused;
    };
    uint32_t uAllGenFlags;
}
GENERATOR_FLAGS;

//
// A move stack
//
typedef struct _MOVE_STACK
{
#ifdef DEBUG
    POSITION board[MAX_PLY_PER_SEARCH];
#endif
    //
    // Unblocked squares map, used for check detection
    //
    uint32_t uUnblockedKeyValue[MAX_PLY_PER_SEARCH];
    KEY_POINTER sUnblocked[MAX_PLY_PER_SEARCH][128];

    //
    // The main move list, a long series of moves, their values and some
    // flag bits to tell search what the values are based upon.
    //
    MOVE_STACK_MOVE_VALUE_FLAGS mvf[MAX_MOVE_STACK];

    //
    // uBegin[ply] and uEnd[ply] specify the start and end of moves gen-
    // erated for a position at ply distance from the root.
    //
    uint32_t uPly;
    uint32_t uBegin[MAX_PLY_PER_SEARCH];
    uint32_t uEnd[MAX_PLY_PER_SEARCH];
    MOVE mvHash[MAX_PLY_PER_SEARCH];
    GENERATOR_FLAGS sGenFlags[MAX_PLY_PER_SEARCH];
}
MOVE_STACK;

#define GENERATE_NO_MOVES                              \
    ctx->sMoveStack.uEnd[ctx->uPly] =                  \
        ctx->sMoveStack.uBegin[ctx->uPly] + 1;         \
    ctx->sMoveStack.uBegin[ctx->uPly + 1] =            \
        ctx->sMoveStack.uEnd[ctx->uPly]

#define GENERATE_ALL_MOVES             (1)
#define GENERATE_ESCAPES               (2)
#define GENERATE_CAPTURES_PROMS_CHECKS (3)
#define GENERATE_CAPTURES_PROMS        (4)
#define GENERATE_DONT_SCORE            (5)
#ifdef TEST
#define GENERATE_ALL_MOVES_CHECK_OK    (6)
#endif

#define MOVE_COUNT(ctx, x)             \
    (((ctx)->sMoveStack.uEnd[(x)]) - (ctx)->sMoveStack.uBegin[(x)])
#define ONE_LEGAL_MOVE(ctx, x) \
    (MOVE_COUNT(ctx, x) == 1)
#define NUM_KING_MOVES(ctx, x) \
    ((ctx)->sMoveStack.sGenFlags[(x)].uKingMoveCount)
#define NUM_CHECKING_PIECES(ctx, x) \
    ((ctx)->sMoveStack.sGenFlags[(x)].uCheckingPieces)


// ----------------------------------------------------------------------
//
// Accumulators
//
typedef struct _COUNTERS
{
    struct
    {
        uint64_t u64Probes;
        uint64_t u64OverallHits;
        uint64_t u64UsefulHits;
        uint64_t u64UpperBoundHits;
        uint64_t u64LowerBoundHits;
        uint64_t u64ExactScoreHits;
    }
    hash;

    struct
    {
        uint64_t u64Probes;
        uint64_t u64Hits;
    }
    pawnhash;

    struct
    {
        uint64_t u64TotalNodeCount;
        uint64_t u64QNodeCount;
        uint64_t u64LeafCount;
        uint64_t u64TerminalPositionCount;
        uint64_t u64BetaCutoffs;
        uint64_t u64BetaCutoffsOnFirstMove;
        uint64_t u64NullMoves;
        uint64_t u64NullMoveSuccess;
#ifdef TEST_NULL
        uint64_t u64QuickNullSuccess;
        uint64_t u64QuickNullDeferredSuccess;
        uint64_t u64QuickNullFailures;
        uint64_t u64AvoidNullSuccess;
        uint64_t u64AvoidNullFailures;
#endif
        uint64_t u64EvalHashHits;
        uint64_t u64LazyEvals;
        uint64_t u64FullEvals;
        uint64_t u64CyclesInEval;
    }
    tree;

    struct
    {
        uint32_t uNumSplits;
        uint32_t uNumSplitsTerminated;
    }
    parallel;

    struct
    {
        uint32_t uPawnPush;
        uint32_t uCheck;
        uint32_t uMateThreat;
        uint32_t uRecapture;
        uint32_t uOneLegalMove;
        uint32_t uNoLegalKingMoves;
        uint32_t uMultiCheck;
        uint32_t uSingularMove;
        uint32_t uZugzwang;
        uint32_t uSingularReplyToCheck;
        uint32_t uEndgame;
        uint32_t uBotvinnikMarkoff;
        uint32_t uQExtend;
    }
    extension;

    struct
    {
        uint32_t uProbes;
        uint32_t uHits;
    }
    egtb;
}
COUNTERS;

typedef struct _CUMULATIVE_SEARCH_FLAGS
{
    // search
    uint32_t uNumChecks[2];                      // not used
    FLAG fInSuspiciousBranch;                 // not used
    FLAG fInReducedDepthBranch;               // not used
    FLAG fAvoidNullmove;                      // restore
    FLAG fVerifyNullmove;                     // restore

    // qsearch
    uint32_t uQsearchDepth;                      // restore
    uint32_t uQsearchNodes;                      // incremental
    uint32_t uQsearchCheckDepth;                 // restore
    FLAG fCouldStandPat[2];                   // restore
}
CUMULATIVE_SEARCH_FLAGS;

// ----------------------------------------------------------------------
//
// Searcher thread record
//
typedef struct _SPLIT_INFO
{
    // locks / counters
    volatile uint32_t uLock;                     // lock for this split node
    volatile uint32_t uNumThreadsHelping;        // num threads in this node
    volatile FLAG fTerminate;                 // signal helpers to terminate

    // moves in the split node
    uint32_t uRemainingMoves;                    // num moves remaining to do
    uint32_t uOnDeckMove;                        // next move to do
    uint32_t uNumMoves;
    uint32_t uAlreadyDone;
    MOVE_STACK_MOVE_VALUE_FLAGS
        mvf[MAX_MOVES_PER_PLY];               // the moves to search

    // input to the split node
    POSITION sRootPosition;                   // the root position
    MOVE mvPathToHere[MAX_PLY_PER_SEARCH];    // path from root to split
    MOVE mvLast;                              // last move before split
    uint32_t uDepth;                             // remaining depth at split
    int32_t iPositionExtend;                      // positional extension
    SCORE iAlpha;                             // original alpha at split
    SCORE iBeta;                              // beta at split
    uint32_t uSplitPositional;                   // pos->uPositional at split
    CUMULATIVE_SEARCH_FLAGS sSearchFlags;     // flags at split time

    // output from the split node
    MOVE mvBest;                              // the best move
    SCORE iBestScore;                         // it's score
    COUNTERS sCounters;                       // updated counters
    MOVE PV[MAX_PLY_PER_SEARCH];

#ifdef DEBUG
    uint32_t uSplitPly;
    POSITION sSplitPosition;
#endif
}
SPLIT_INFO;

typedef struct _PLY_INFO
{
#ifdef DEBUG
    POSITION sPosition;
    SCORE iAlpha;
    SCORE iBeta;
#endif
    SCORE iEval;
    int32_t iExtensionAmount;
    FLAG fInCheck;
    FLAG fInQsearch;
    MOVE mv;
    MOVE mvBest;
    MOVE PV[MAX_PLY_PER_SEARCH];

    SCORE iKingScore[2];
    uint32_t uMinMobility[2];
    uint64_t u64NonPawnSig;
    uint64_t u64PawnSig;
    uint64_t u64Sig;
    uint32_t uFifty;
    FLAG fCastled[2];
    uint32_t uTotalNonPawns;
    BITV bvCastleInfo;
    COOR cEpSquare;
}
PLY_INFO;

#define EVAL_HASH
#ifdef EVAL_HASH
#if defined(_ANDROID)
#define EVAL_HASH_TABLE_SIZE (100) 
#else
#define EVAL_HASH_TABLE_SIZE (2097152) // 32Mb (per thread)
#endif
typedef struct _EVAL_HASH_ENTRY
{
    uint64_t u64Key;
    SCORE iEval;
    uint32_t uPositional;
    COOR cTrapped[2];

} EVAL_HASH_ENTRY;
#endif
#if defined(_ANDROID)
#define PAWN_HASH_TABLE_SIZE (100) // 5.5Mb (per thread)
#else
#define PAWN_HASH_TABLE_SIZE (131072) // 5.5Mb (per thread)
#endif
typedef struct _PAWN_HASH_ENTRY
{
    uint64_t u64Key;
    BITBOARD bbPawnLocations[2];
    BITBOARD bbPasserLocations[2];
    BITBOARD bbStationaryPawns[2];
    int16_t iScore[2];
    uint8_t uCountPerFile[2][10];
    uint8_t uNumRammedPawns;
    uint8_t uNumUnmovedPawns[2];
}
PAWN_HASH_ENTRY;

#define NUM_SPLIT_PTRS_IN_CONTEXT (8)

typedef struct _SEARCHER_THREAD_CONTEXT
{
    uint32_t uPly;                               // its distance from root
    uint32_t uPositional;                        // positional component of score
    POSITION sPosition;                       // the board
    MOVE_STACK sMoveStack;                    // the move stack
    CUMULATIVE_SEARCH_FLAGS sSearchFlags;
    PLY_INFO sPlyInfo[MAX_PLY_PER_SEARCH+1];
    MOVE mvKiller[MAX_PLY_PER_SEARCH][2];
    MOVE mvKillerEscapes[MAX_PLY_PER_SEARCH][2];
    MOVE mvNullmoveRefutations[MAX_PLY_PER_SEARCH];
    COUNTERS sCounters;
    uint32_t uThreadNumber;
    SPLIT_INFO *pSplitInfo[NUM_SPLIT_PTRS_IN_CONTEXT];
    MOVE mvRootMove;
    SCORE iRootScore;
    uint32_t uRootDepth;
    PAWN_HASH_ENTRY rgPawnHash[PAWN_HASH_TABLE_SIZE];
#ifdef EVAL_HASH
    EVAL_HASH_ENTRY rgEvalHash[EVAL_HASH_TABLE_SIZE];
#endif
    int8_t szLastPV[SMALL_STRING_LEN_CHAR];
}
SEARCHER_THREAD_CONTEXT;

//
// When I added the pawn hash table and eval hash tables to searcher
// thread contexts they became too heavy to just allocate on the fly
// for things like checking the legality of SAN moves or seeing if an
// opening book line leads to a draw.  This
// LIGHTWEIGHT_SEARCHER_CONTEXT structure can be cast into a full
// SEARCHER_THREAD_CONTEXT and passed safely into the Generate,
// MakeMove and UnMakeMove functions because they presently only
// need:
//
//     1. uPly
//     2. sPosition
//     3. sPlyInfo
//     4. sMoveStack
//
typedef struct _LIGHTWEIGHT_SEARCHER_CONTEXT
{
    uint32_t uPly;
    uint32_t uPositional;
    POSITION sPosition;
    MOVE_STACK sMoveStack;
    CUMULATIVE_SEARCH_FLAGS sSearchFlags;
    PLY_INFO sPlyInfo[MAX_PLY_PER_SEARCH+1];
}
LIGHTWEIGHT_SEARCHER_CONTEXT;

// ----------------------------------------------------------------------
//
// Global game options
//
typedef struct _GAME_OPTIONS
{
    uint32_t uMyClock;
    uint32_t uOpponentsClock;
    FLAG fGameIsRated;
    FLAG fOpponentIsComputer;
    uint32_t uSecPerMove;
    uint32_t uMaxDepth;
    uint64_t u64MaxNodeCount;
    FLAG fShouldPonder;
    FLAG fPondering;
    FLAG fThinking;
    FLAG fSuccessfulPonder;
    MOVE mvPonder;
    FLAG fShouldPost;
    FLAG fForceDrawWorthZero;
    uint32_t uMyIncrement;
    uint32_t uMovesPerTimePeriod;
    int8_t szAnalyzeProgressReport[SMALL_STRING_LEN_CHAR];
    FLAG fShouldAnnounceOpening;
    SCORE iLastEvalScore;
    uint64_t u64NodesSearched;
    int8_t szLogfile[SMALL_STRING_LEN_CHAR];
    int8_t szEGTBPath[SMALL_STRING_LEN_CHAR];
    int8_t szBookName[SMALL_STRING_LEN_CHAR];
    uint32_t uNumProcessors;
    uint32_t uNumHashTableEntries;
    FLAG fNoInputThread;
    FLAG fVerbosePosting;
    FLAG fRunningUnderXboard;
    FLAG fStatusLine;
    FLAG fFastScript;
    int32_t iResignThreshold;

    enum
    {
        CLOCK_NORMAL               = 0,
        CLOCK_FIXED,
        CLOCK_INCREMENT,
        CLOCK_NONE
    }
    eClock;

    enum
    {
        GAME_UNKNOWN               = 0,
        GAME_BULLET,
        GAME_BLITZ,
        GAME_STANDARD
    }
    eGameType;

    enum
    {
        I_PLAY_WHITE               = 0,
        I_PLAY_BLACK,
        FORCE_MODE,
        EDIT_MODE,
        ANALYZE_MODE
    }
    ePlayMode;
}
GAME_OPTIONS;
extern GAME_OPTIONS g_Options;

// ----------------------------------------------------------------------
//
// Move timer
//

#define TIMER_SEARCHING_FIRST_MOVE       (0x1)
#define TIMER_SEARCHING_IMPORTANT_MOVE   (0x2)
#define TIMER_RESOLVING_ROOT_FL          (0x4)
#define TIMER_RESOLVING_ROOT_FH          (0x8)
#define TIMER_JUST_OUT_OF_BOOK           (0x10)
#define TIMER_CURRENT_OBVIOUS            (0x20)
#define TIMER_CURRENT_WONT_UNBLOCK       (0x40)
#define TIMER_ROOT_POSITION_CRITICAL     (0x80)
#define TIMER_MOVE_IMMEDIATELY           (0x100)
#define TIMER_MANY_ROOT_FLS              (0x200)
#define TIMER_STOPPING                   (0x400)
#define TIMER_SPLIT_FAILED               (0x800)

typedef struct _MOVE_TIMER
{
    double dStartTime;
    double dEndTime;
    double dSoftTimeLimit;
    double dHardTimeLimit;
    uint32_t uNodeCheckMask;
    volatile BITV bvFlags;
}
MOVE_TIMER;

// ----------------------------------------------------------------------
//
// Special move tag literals
//
#define ILLEGALMOVE                ((uint32_t)0x1DDD8888)

//
// useful macros
//
#ifdef DEBUG
void
_assert(int8_t *szFile, uint32_t uLine);

#define ASSERT(x)                  if (x)    \
                                   { ; }     \
                                   else      \
                                   { _assert(__FILE__, __LINE__); }
#define VERIFY(x)                  ASSERT(x)
#else
#define ASSERT(x)                   ;
#define VERIFY(x)                  x;
#endif // DEBUG

#ifdef PERF_COUNTERS
#define INC(x)                     ((x) += 1)
#else
#define INC(x)
#endif

#define BREAKPOINT                 SystemDebugBreakpoint()

#define MIN(x, y)                  (((x) < (y)) ? (x) : (y))
#define MAX(x, y)                  (((x) > (y)) ? (x) : (y))

#ifdef _X86_
//
// Note: MAXU, MINU and ABS_DIFF require arguments with the high order
// bit CLEAR to work right.
//
// These are branchless constructs.  MAXU and MINU are equivalent to
// MIN and MAX (with the above restriction on inputs)
//
// MIN0 and MAX0 are equivalent to MAX(0, x) and MIN(0, x).  These
// macros have no restiction on argument type.
//
// ABS_DIFF is equivalent to abs(x - y).  Again, x and y must have
// their high-order bits CLEAR for this to work.
//
// Note: gcc generates code with cmovs so no need for MAXU/MINU on
// that compiler.
//
#define MINU(x, y)                 \
    (((((int32_t)((x)-(y)))>>31) & ((x)-(y)))+(y))
#define MAXU(x, y)                 \
    (((((int32_t)((x)-(y)))>>31) & ((y)-(x)))+(x))
#define MIN0(x)                    \
    ((x) & (((int32_t)(x)) >> 31))
#define MAX0(x)                    \
    ((x) & ~(((int32_t)(x)) >> 31))


#define ABS_DIFF(a, b)             \
    (((b)-(a)) - ((((b) - (a)) & (((int32_t)((b) - (a))) >> 31) ) << 1))

#endif // _X86_

#ifndef MINU
#define MINU(x, y)                 (MIN((x), (y)))
#endif

#ifndef MAXU
#define MAXU(x, y)                 (MAX((x), (y)))
#endif

#ifndef MIN0
#define MIN0(x)                    (MIN((x), 0))
#endif

#ifndef MAX0
#define MAX0(x)                    (MAX((x), 0))
#endif

#ifndef ABS_DIFF
#define ABS_DIFF(a, b)             (abs((a) - (b)))
#endif

#define FILE_DISTANCE(a, b)        (ABS_DIFF(MYFILE((a)), MYFILE((b))))
#define RANK_DISTANCE(a, b)        (ABS_DIFF(((a) & 0xF0), ((b) & 0xF0)) >> 4)
#define REAL_DISTANCE(a, b)        (MAXU(FILE_DISTANCE((a), (b)), \
                                         RANK_DISTANCE((a), (b))))
#ifdef DEBUG
#define DISTANCE(a, b)             DistanceBetweenSquares((a), (b))
#else
#define DISTANCE(a, b)             g_pDistance[(a) - (b)]
#endif // DEBUG

#define    IS_EMPTY( square )      (!(square))
#define IS_OCCUPIED( square )      ((square))

#define IS_DEAD( listindex )       ((listindex) > 119)
#define IS_ALIVE( listindex )      ((listindex) <= 119)
#define ILLEGAL_COOR               (0x88)
#define EDGE_DISTANCE(c)           (MIN(MIN(abs(RANK(c) - 7), RANK(c)), \
                                        MIN(abs(MYFILE(c) - 7), MYFILE(c))))
#define NOT_ON_EDGE(c)             ((((c) & 0xF0) != 0x00) && \
                                       (((c) & 0xF0) != 0x70) && \
                                       (((c) & 0x0F) != 0x00) && \
                                       (((c) & 0x0F) != 0x07))
#define ON_EDGE(c)                 (RANK8(c) || \
                                    RANK1(c) || \
                                    FILEA(c) || \
                                    FILEH(c))
#define CORNER_DISTANCE(c)         (MAX(MIN((uint32_t)abs(RANK(c) - 7), RANK(c)),\
                                        MIN((uint32_t)abs(MYFILE(c) - 7), MYFILE(c))))
#define IN_CORNER(c)               (((c) == A8) || \
                                       ((c) == A1) || \
                                       ((c) == H8) || \
                                       ((c) == H1))
#define WHITE_CORNER_DISTANCE(c)   (MIN(DISTANCE((c), A8), \
                                        DISTANCE((c), H1)))
#define BLACK_CORNER_DISTANCE(c)   (MIN(DISTANCE((c), H8), \
                                        DISTANCE((c), A1)))
#define SYM_SQ(c)                  ((7 - (((c) & 0xF0) >> 4)) << 4) \
                                       | (7 - ((c) & 0x0F))
#define ARRAY_SIZE(a)              (sizeof((a)) / sizeof((a[0])))
#define MAKE_PSQT(a, b, c, d)      (((a) << 24) | ((b) << 16) | \
                                       ((c) << 8) | (d))
#define TO64(x)                    ((x) & 0x7) + ((0x7 - ((x)>>4)) << 3)
#define COOR_TO_BIT_NUMBER(c)      (((((c) & 0x70) >> 1) | ((c) & 0x7)))
#define SLOWCOOR_TO_BB(c)          (1ULL << COOR_TO_BIT_NUMBER(c))
#define COOR_TO_BB(c)              (BBSQUARE[COOR_TO_BIT_NUMBER(c)])
#define SLOW_BIT_NUMBER_TO_COOR(b) ((((b) / 8) << 4) + ((b) & 7))
#define BIT_NUMBER_TO_COOR(b)      ((((b) & 0xF8) << 1) | ((b) & 7))

#define BBRANK88 \
    SLOWCOOR_TO_BB(A8) | SLOWCOOR_TO_BB(B8) | SLOWCOOR_TO_BB(C8) | \
    SLOWCOOR_TO_BB(D8) | SLOWCOOR_TO_BB(E8) | SLOWCOOR_TO_BB(F8) | \
    SLOWCOOR_TO_BB(G8) | SLOWCOOR_TO_BB(H8)

#define BBRANK77 \
    SLOWCOOR_TO_BB(A7) | SLOWCOOR_TO_BB(B7) | SLOWCOOR_TO_BB(C7) | \
    SLOWCOOR_TO_BB(D7) | SLOWCOOR_TO_BB(E7) | SLOWCOOR_TO_BB(F7) | \
    SLOWCOOR_TO_BB(G7) | SLOWCOOR_TO_BB(H7)

#define BBRANK66 \
    SLOWCOOR_TO_BB(A6) | SLOWCOOR_TO_BB(B6) | SLOWCOOR_TO_BB(C6) | \
    SLOWCOOR_TO_BB(D6) | SLOWCOOR_TO_BB(E6) | SLOWCOOR_TO_BB(F6) | \
    SLOWCOOR_TO_BB(G6) | SLOWCOOR_TO_BB(H6)

#define BBRANK55 \
    SLOWCOOR_TO_BB(A5) | SLOWCOOR_TO_BB(B5) | SLOWCOOR_TO_BB(C5) | \
    SLOWCOOR_TO_BB(D5) | SLOWCOOR_TO_BB(E5) | SLOWCOOR_TO_BB(F5) | \
    SLOWCOOR_TO_BB(G5) | SLOWCOOR_TO_BB(H5)

#define BBRANK44 \
    SLOWCOOR_TO_BB(A4) | SLOWCOOR_TO_BB(B4) | SLOWCOOR_TO_BB(C4) | \
    SLOWCOOR_TO_BB(D4) | SLOWCOOR_TO_BB(E4) | SLOWCOOR_TO_BB(F4) | \
    SLOWCOOR_TO_BB(G4) | SLOWCOOR_TO_BB(H4)

#define BBRANK33 \
    SLOWCOOR_TO_BB(A3) | SLOWCOOR_TO_BB(B3) | SLOWCOOR_TO_BB(C3) | \
    SLOWCOOR_TO_BB(D3) | SLOWCOOR_TO_BB(E3) | SLOWCOOR_TO_BB(F3) | \
    SLOWCOOR_TO_BB(G3) | SLOWCOOR_TO_BB(H3)

#define BBRANK22 \
    SLOWCOOR_TO_BB(A2) | SLOWCOOR_TO_BB(B2) | SLOWCOOR_TO_BB(C2) | \
    SLOWCOOR_TO_BB(D2) | SLOWCOOR_TO_BB(E2) | SLOWCOOR_TO_BB(F2) | \
    SLOWCOOR_TO_BB(G2) | SLOWCOOR_TO_BB(H2)

#define BBRANK11 \
    SLOWCOOR_TO_BB(A1) | SLOWCOOR_TO_BB(B1) | SLOWCOOR_TO_BB(C1) | \
    SLOWCOOR_TO_BB(D1) | SLOWCOOR_TO_BB(E1) | SLOWCOOR_TO_BB(F1) | \
    SLOWCOOR_TO_BB(G1) | SLOWCOOR_TO_BB(H1)

#define BBFILEA \
    SLOWCOOR_TO_BB(A1) | SLOWCOOR_TO_BB(A2) | SLOWCOOR_TO_BB(A3) | \
    SLOWCOOR_TO_BB(A4) | SLOWCOOR_TO_BB(A5) | SLOWCOOR_TO_BB(A6) | \
    SLOWCOOR_TO_BB(A7) | SLOWCOOR_TO_BB(A8)

#define BBFILEB \
    SLOWCOOR_TO_BB(B1) | SLOWCOOR_TO_BB(B2) | SLOWCOOR_TO_BB(B3) | \
    SLOWCOOR_TO_BB(B4) | SLOWCOOR_TO_BB(B5) | SLOWCOOR_TO_BB(B6) | \
    SLOWCOOR_TO_BB(B7) | SLOWCOOR_TO_BB(B8)

#define BBFILEC \
    SLOWCOOR_TO_BB(C1) | SLOWCOOR_TO_BB(C2) | SLOWCOOR_TO_BB(C3) | \
    SLOWCOOR_TO_BB(C4) | SLOWCOOR_TO_BB(C5) | SLOWCOOR_TO_BB(C6) | \
    SLOWCOOR_TO_BB(C7) | SLOWCOOR_TO_BB(C8)

#define BBFILED \
    SLOWCOOR_TO_BB(D1) | SLOWCOOR_TO_BB(D2) | SLOWCOOR_TO_BB(D3) | \
    SLOWCOOR_TO_BB(D4) | SLOWCOOR_TO_BB(D5) | SLOWCOOR_TO_BB(D6) | \
    SLOWCOOR_TO_BB(D7) | SLOWCOOR_TO_BB(D8)

#define BBFILEE \
    SLOWCOOR_TO_BB(E1) | SLOWCOOR_TO_BB(E2) | SLOWCOOR_TO_BB(E3) | \
    SLOWCOOR_TO_BB(E4) | SLOWCOOR_TO_BB(E5) | SLOWCOOR_TO_BB(E6) | \
    SLOWCOOR_TO_BB(E7) | SLOWCOOR_TO_BB(E8)

#define BBFILEF \
    SLOWCOOR_TO_BB(F1) | SLOWCOOR_TO_BB(F2) | SLOWCOOR_TO_BB(F3) | \
    SLOWCOOR_TO_BB(F4) | SLOWCOOR_TO_BB(F5) | SLOWCOOR_TO_BB(F6) | \
    SLOWCOOR_TO_BB(F7) | SLOWCOOR_TO_BB(F8)

#define BBFILEG \
    SLOWCOOR_TO_BB(G1) | SLOWCOOR_TO_BB(G2) | SLOWCOOR_TO_BB(G3) | \
    SLOWCOOR_TO_BB(G4) | SLOWCOOR_TO_BB(G5) | SLOWCOOR_TO_BB(G6) | \
    SLOWCOOR_TO_BB(G7) | SLOWCOOR_TO_BB(G8)

#define BBFILEH \
    SLOWCOOR_TO_BB(H1) | SLOWCOOR_TO_BB(H2) | SLOWCOOR_TO_BB(H3) | \
    SLOWCOOR_TO_BB(H4) | SLOWCOOR_TO_BB(H5) | SLOWCOOR_TO_BB(H6) | \
    SLOWCOOR_TO_BB(H7) | SLOWCOOR_TO_BB(H8)

#define BBRANK72 \
    BBRANK77 | BBRANK66 | BBRANK55 | BBRANK44 | BBRANK33 | BBRANK22

#define BBRANK62 \
    BBRANK66 | BBRANK55 | BBRANK44 | BBRANK33 | BBRANK22

#define BBRANK52 \
    BBRANK55 | BBRANK44 | BBRANK33 | BBRANK22

#define BBRANK42 \
    BBRANK44 | BBRANK33 | BBRANK22

#define BBRANK32 \
    BBRANK33 | BBRANK22

#define BBRANK27 \
    BBRANK22 | BBRANK33 | BBRANK44 | BBRANK55 | BBRANK66 | BBRANK77

#define BBRANK37 \
    BBRANK33 | BBRANK44 | BBRANK55 | BBRANK66 | BBRANK77

#define BBRANK47 \
    BBRANK44 | BBRANK55 | BBRANK66 | BBRANK77

#define BBRANK57 \
    BBRANK55 | BBRANK66 | BBRANK77

#define BBRANK67 \
    BBRANK66 | BBRANK77

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

//
// util.c
//
COMMAND(LearnPsqtFromPgn);

COMMAND(GeneratePositionAndBestMoveSuite);

int8_t *
WalkPV(SEARCHER_THREAD_CONTEXT *ctx);

int8_t *
ReadNextGameFromPgnFile(FILE *pf);

COOR
DistanceBetweenSquares(COOR a, COOR b);

void CDECL
Trace(int8_t *szMessage, ...);

void CDECL
Log(int8_t *szMessage, ...);

void CDECL
Bug(int8_t *szMessage, ...);

int8_t *
FindChunk(int8_t *sz, uint32_t uTargetChunk);

FLAG
BufferIsZeroed(uint8_t *p, uint32_t u);

int8_t *
ColorToString(uint32_t u);

int8_t *
CoorToString(COOR c);

int8_t *
ScoreToString(SCORE iScore);

int8_t *
TimeToString(double d);

int32_t
AcquireSpinLock(volatile int32_t *pSpinlock);

FLAG
TryAcquireSpinLock(volatile int32_t *pSpinlock);

void
ReleaseSpinLock(volatile int32_t *pSpinLock);

uint8_t
Checksum(uint8_t *p, uint32_t uSize);

FLAG
BackupFile(int8_t *szFile);

void
UtilPrintPV(SEARCHER_THREAD_CONTEXT *ctx,
            SCORE iAlpha,
            SCORE iBeta,
            SCORE iScore,
            MOVE mv);

#define CANNOT_INITIALIZE_SPLIT           (1)
#define INCONSISTENT_POSITION             (2)
#define UNEXPECTED_SYSTEM_CALL_FAILURE    (3)
#define SHOULD_NOT_GET_HERE               (4)
#define GOT_ILLEGAL_MOVE_WHILE_PONDERING  (5)
#define CANNOT_OFFICIALLY_MAKE_MOVE       (6)
#define DETECTED_INCORRECT_INITIALIZATION (7)
#define INCONSISTENT_STATE                (8)
#define FATAL_ACCESS_DENIED               (9)
#define TESTCASE_FAILURE                  (10)
#define INITIALIZATION_FAILURE            (11)

void
UtilPanic(uint32_t uPanicCode,
          POSITION *pos,
          void *a1,
          void *a2,
          void *a3,
          int8_t *file, uint32_t line);


//
// main.c
//
#define LOGFILE_NAME                "typhoon.log"
extern FILE *g_pfLogfile;

void
Banner(void);

FLAG
PreGameReset(FLAG fResetBoard);

//
// system dependent exports (see win32.c or fbsd.c)
//
typedef uint32_t (THREAD_ENTRYPOINT)(uint32_t);

#ifdef DEBUG
uint32_t
GetHeapMemoryUsage(void);
#endif

void
SystemDebugBreakpoint(void);

int8_t *
SystemStrDup(int8_t *p);

double
SystemTimeStamp(void);

FLAG
SystemDoesFileExist(int8_t *szFilename);

void
SystemDeferExecution(uint32_t uMs);

void
SystemFreeMemory(void *pMem);

void *
SystemAllocateMemory(uint32_t dwSizeBytes);

FLAG
SystemMakeMemoryReadOnly(void *pMemory, uint32_t dwSizeBytes);

FLAG
SystemMakeMemoryNoAccess(void *pMemory, uint32_t dwSizeBytes);

FLAG
SystemMakeMemoryReadWrite(void *pMemory, uint32_t dwSizeBytes);

FLAG
SystemDependentInitialization(void);

uint64_t FASTCALL
SystemReadTimeStampCounter(void);

//
// This routine _must_ implement a full memory fence -- it is used in
// util.c to implement spinlocks.
//
FLAG
SystemCreateThread(THREAD_ENTRYPOINT *pEntry, uint32_t uParam, uint32_t *puHandle);

FLAG
SystemWaitForThreadToExit(uint32_t uThreadHandle);

FLAG
SystemGetThreadExitCode(uint32_t uThreadHandle, uint32_t *puCode);

FLAG
SystemDeleteThread(uint32_t uThreadHandle);

int8_t *
SystemGetDateString(void);

int8_t *
SystemGetTimeString(void);

FLAG
SystemCopyFile(int8_t *szSource, int8_t *szDest);

FLAG
SystemDeleteFile(int8_t *szFile);

uint32_t
SystemCreateLock(void);

FLAG
SystemDeleteLock(uint32_t u);

FLAG
SystemBlockingWaitForLock(uint32_t u);

FLAG
SystemReleaseLock(uint32_t u);

uint32_t
SystemCreateSemaphore(uint32_t u);

FLAG
SystemDeleteSemaphore(uint32_t u);

void
SystemReleaseSemaphoreResource(uint32_t u);

void
SystemObtainSemaphoreResource(uint32_t u);


//
// fen.c
//
#define STARTING_POSITION_IN_FEN \
    "rnbqkbnr/pppppppp/--------/8/8/--------/PPPPPPPP/RNBQKBNR w KQkq - 0 1"

FLAG
LooksLikeFen(int8_t *szFen);

int8_t *
PositionToFen(POSITION *p);

FLAG
FenToPosition(POSITION *p, int8_t *szFen);

//
// testfen.c
//
#ifdef TEST
void
TestFenCode(void);
#endif

//
// piece.c
//
// INVERTED VALUE == (VALUE_QUEEN / VALUE_PIECE) * VALUE_PAWN;
#define VALUE_PAWN            100
#define INVERT_PAWN           900
#define VALUE_KNIGHT          300
#define INVERT_KNIGHT         300
#define VALUE_BISHOP          300
#define INVERT_BISHOP         300
#define VALUE_ROOK            500
#define INVERT_ROOK           180
#define VALUE_QUEEN           975
#define INVERT_QUEEN          100
#define VALUE_KING            (INFINITY)
#define INVERT_KING           1

#define VALUE_FULL_ARMY       (8 * VALUE_PAWN) + (2 * VALUE_KNIGHT) + \
                              (2 * VALUE_BISHOP) + (2 * VALUE_ROOK) + \
                              VALUE_QUEEN + VALUE_KING
#define VALUE_MAX_ARMY        (9 * VALUE_QUEEN) + (2 * VALUE_KNIGHT) + \
                              (2 * VALUE_BISHOP) + (2 * VALUE_ROOK) + \
                              VALUE_KING

typedef struct _PIECE_DATA
{
    uint32_t uValue;
    uint32_t uValueOver100;
    uint32_t uInvertedValue;
    int8_t *szName;
} PIECE_DATA;

extern PIECE_DATA g_PieceData[8];

#define PIECE_VALUE_OVER_100(p) (g_PieceData[PIECE_TYPE(p)].uValueOver100)
extern uint32_t
PieceValueOver100(PIECE p);

#define PIECE_VALUE(p)          (g_PieceData[PIECE_TYPE(p)].uValue)
extern uint32_t
PieceValue(PIECE p);

#define INVERTED_PIECE_VALUE(p) (g_PieceData[PIECE_TYPE(p)].uInvertedValue)
extern uint32_t
PieceInvertedValue(PIECE p);

int8_t *
PieceAbbrev(PIECE p);

//
// board.c
//
FLAG
VerifyPositionConsistency(POSITION *pos, FLAG fContinueOnError);

FLAG
PositionsAreEquivalent(POSITION *p1, POSITION *p2);

int8_t *
DrawTextBoardFromPosition(POSITION *pos);

void
DumpPosition(POSITION *pos);

int8_t *
CastleInfoString(BITV bv);

void
SetRootToInitialPosition(void);

//
// move.c
//
void
SlidePiece(POSITION *pos, COOR cFrom, COOR cTo);

PIECE
LiftPiece(POSITION *pos, COOR cSquare);

void
PlacePiece(POSITION *pos, COOR cSquare, PIECE pPiece);

FLAG
MakeMove(SEARCHER_THREAD_CONTEXT *ctx,
         MOVE mv);

void
UnmakeMove(SEARCHER_THREAD_CONTEXT *ctx,
           MOVE mv);

FLAG
MakeUserMove(SEARCHER_THREAD_CONTEXT *ctx, MOVE mvUser);

void
DumpMove(uint32_t u);

//
// movesup.c
//
#define MOVE_TO_INDEX(mv)  (((mv).uMove & 0xFFFF) + \
                            (0x10000 * GET_COLOR(mv.pMoved)))

COOR
FasterExposesCheck(POSITION *pos,
                   COOR cRemove,
                   COOR cLocation);

COOR
ExposesCheck(POSITION *pos,
             COOR cRemove,
             COOR cLocation);

COOR
ExposesCheckEp(POSITION *pos,
               COOR cTest,
               COOR cIgnore,
               COOR cBlock,
               COOR cKing);

FLAG
IsAttacked(COOR cTest, POSITION *pos, uint32_t uSide);

FLAG
InCheck(POSITION *pos, uint32_t uSide);

FLAG
SanityCheckMove(POSITION *pos, MOVE mv);

FLAG
LooksLikeFile(int8_t c);

FLAG
LooksLikeRank(int8_t c);

FLAG
LooksLikeCoor(int8_t *szData);

int8_t *
StripMove(int8_t *szMove);

uint32_t
LooksLikeMove(int8_t *szData);

void FASTCALL
SelectBestWithHistory(SEARCHER_THREAD_CONTEXT *ctx, uint32_t u);

void FASTCALL
SelectBestNoHistory(SEARCHER_THREAD_CONTEXT *ctx, uint32_t u);

void FASTCALL
SelectMoveAtRoot(SEARCHER_THREAD_CONTEXT *ctx, uint32_t u);

#define NOT_MOVE 0
#define MOVE_ICS 1
#define MOVE_SAN 2

COMMAND(PerftCommand);


//
// testmove.c
//
#ifdef TEST
void
TestLiftPlaceSlidePiece(void);

void
TestExposesCheck(void);

void
TestIsAttacked(void);

void
TestMakeUnmakeMove(void);
#endif

//
// generate.c
//
#define SORT_THESE_FIRST (0x40000000)
#define FIRST_KILLER     (0x20000000)
#define SECOND_KILLER    (0x10000000)
#define THIRD_KILLER     (0x08000000)
#define FOURTH_KILLER    (0x04000000)
#define GOOD_MOVE        (0x02000000)
#define STRIP_OFF_FLAGS  (0x00FFFFFF)

extern const int32_t g_iQKDeltas[9];
extern const int32_t g_iNDeltas[9];
extern const int32_t g_iBDeltas[5];
extern const int32_t g_iRDeltas[5];

void
GenerateMoves(SEARCHER_THREAD_CONTEXT *ctx,
              MOVE mvOrderFirst,
              uint32_t uType);

FLAG
WouldGiveCheck(IN SEARCHER_THREAD_CONTEXT *ctx,
               IN MOVE mv);

//
// testgenerate.c
//
#ifdef TEST

void
PlyTest(SEARCHER_THREAD_CONTEXT *ctx,
        uint32_t uDepth,
        FLAG fRootPositionInCheck);

void
TestMoveGenerator(void);

void
TestLegalMoveGenerator(void);

#endif

//
// sig.c
//
extern uint64_t g_u64SigSeeds[128][7][2];
extern uint64_t g_u64PawnSigSeeds[128][2];
extern uint64_t g_u64CastleSigSeeds[16];
extern uint64_t g_u64EpSigSeeds[9];

void
InitializeSigSystem(void);

uint64_t
ComputePawnSig(POSITION *pos);

uint64_t
ComputeSig(POSITION *pos);

//
// mersenne.c
//
void
seedMT(uint32_t seed);

uint32_t
reloadMT(void);

uint32_t
randomMT(void);

//
// data.c
//
typedef struct _VECTOR_DELTA
{
    uint8_t iVector[2];
     int8_t iDelta;
     int8_t iNegDelta;
}
VECTOR_DELTA;

extern uint32_t g_uDistance[256];
extern uint32_t *g_pDistance;
extern VECTOR_DELTA g_VectorDelta[256];
extern VECTOR_DELTA *g_pVectorDelta;
extern int8_t g_SwapTable[14][32][32];
extern SCORE _PSQT[14][128];
extern uint32_t g_uSearchSortLimits[];
extern MOVE NULLMOVE;
extern MOVE HASHMOVE;
extern MOVE RECOGNMOVE;
extern MOVE DRAWMOVE;
extern MOVE MATEMOVE;
extern FLAG g_fIsWhiteSquare[128];
extern BITBOARD BBFILE[8];
extern BITBOARD BBRANK[9];
extern BITBOARD BBWHITESQ;
extern BITBOARD BBBLACKSQ;
extern BITBOARD BBLEFTSIDE;
extern BITBOARD BBRIGHTSIDE;
extern BITBOARD BBSQUARE[64];
extern BITBOARD BBROOK_PAWNS;
extern BITBOARD BBPRECEEDING_RANKS[8][2];
extern BITBOARD BBADJACENT_FILES[8];
extern BITBOARD BBADJACENT_RANKS[9];

void
InitializeWhiteSquaresTable(void);

void
InitializeVectorDeltaTable(void);

void
InitializeSwapTable(void);

void
InitializeDistanceTable(void);

void
InitializeSearchDepthArray(void);

uint32_t
GetSearchSortLimit(uint32_t);

#ifdef DEBUG
#define SEARCH_SORT_LIMIT(x)    (GetSearchSortLimit((x)))
#else
#define SEARCH_SORT_LIMIT(x)    (g_uSearchSortLimits[(x)])
#endif

#ifdef DEBUG
uint32_t CheckVectorWithIndex(int32_t i, uint32_t uColor);
#define CHECK_VECTOR_WITH_INDEX(i, color) \
    CheckVectorWithIndex(i, color)

int32_t DirectionBetweenSquaresWithIndex(int32_t i);
#define CHECK_DELTA_WITH_INDEX(i) \
    DirectionBetweenSquaresWithIndex(i)

int32_t DirectionBetweenSquaresFromTo(COOR, COOR);
#define DIRECTION_BETWEEN_SQUARES(from, to) \
    DirectionBetweenSquaresFromTo(from, to)

int32_t NegativeDirectionBetweenSquaresWithIndex(int32_t i);
#define NEG_DELTA_WITH_INDEX(i) \
    NegativeDirectionBetweenSquaresWithIndex(i)

FLAG IsSquareWhite(COOR c);
#define IS_SQUARE_WHITE(c) \
    IsSquareWhite(c)

#else
#define CHECK_VECTOR_WITH_INDEX(i, color) \
    (g_pVectorDelta[(i)].iVector[(color)])

#define CHECK_DELTA_WITH_INDEX(i) \
    (g_pVectorDelta[(i)].iDelta)

#define DIRECTION_BETWEEN_SQUARES(cFrom, cTo) \
    CHECK_DELTA_WITH_INDEX((int32_t)(cFrom) - (int32_t)(cTo))

#define NEG_DELTA_WITH_INDEX(i) \
    (g_pVectorDelta[(i)].iNegDelta)

#define IS_SQUARE_WHITE(c) \
    (g_fIsWhiteSquare[(c)])
#endif

//
// san.c
//
MOVE
ParseMoveSan(int8_t *szInput,
             POSITION *pos);

int8_t *
MoveToSan(MOVE mv, POSITION *pos);

//
// testsan.c
//
void
TestSan(void);

//
// list.c
//
void
InitializeListHead(IN DLIST_ENTRY *pListHead);

FLAG
IsListEmpty(IN DLIST_ENTRY *pListHead);

FLAG
RemoveEntryList(IN DLIST_ENTRY *pEntry);

DLIST_ENTRY *
RemoveHeadList(IN DLIST_ENTRY *pListHead);

DLIST_ENTRY *
RemoveTailList(IN DLIST_ENTRY *pListHead);

void
InsertTailList(IN DLIST_ENTRY *pListHead,
               IN DLIST_ENTRY *pEntry);

void
InsertHeadList(IN DLIST_ENTRY *pListHead,
               IN DLIST_ENTRY *pEntry);

//
// command.c
//
void
ParseUserInput(FLAG fSearching);

FLAG
InitializeCommandSystem(void);

void
CleanupCommandSystem(void);

//
// input.c
//
void
InitInputSystemInBatchMode(void);

uint32_t
InitInputSystemWithDedicatedThread(void);

void
PushNewInput(int8_t *buf);

int8_t *
PeekNextInput(void);

int8_t *
ReadNextInput(void);

int8_t *
BlockingReadInput(void);

uint32_t
NumberOfPendingInputEvents(void);

volatile extern FLAG g_fExitProgram;

//
// ics.c
//
MOVE
ParseMoveIcs(int8_t *szInput, POSITION *pos);

int8_t *
MoveToIcs(MOVE mv);

//
// testics.c
//
void
TestIcs(void);

//
// gamelist.c
//
typedef enum _ERESULT
{
    RESULT_BLACK_WON = -1,
    RESULT_DRAW = 0,
    RESULT_WHITE_WON = 1,
    RESULT_IN_PROGRESS,
    RESULT_ABANDONED,
    RESULT_UNKNOWN
} ERESULT;

typedef struct _GAME_RESULT
{
    ERESULT eResult;
    int8_t szDescription[256];
} GAME_RESULT;

typedef struct _GAME_PLAYER
{
    int8_t *szName;
    int8_t *szDescription;
    FLAG fIsComputer;
    uint32_t uRating;
}
GAME_PLAYER;

typedef struct _GAME_HEADER
{
    int8_t *szGameDescription;
    int8_t *szLocation;
    GAME_PLAYER sPlayer[2];
    FLAG fGameIsRated;
    uint64_t u64OpeningSig;
    GAME_RESULT result;
    int8_t *szInitialFen;
}
GAME_HEADER;

typedef struct _GAME_MOVE
{
    DLIST_ENTRY links;
    uint32_t uNumber;
    MOVE mv;
    int8_t *szComment;
    int8_t *szDecoration;
    int8_t *szMoveInSan;
    int8_t *szMoveInIcs;
    int8_t *szUndoPositionFen;
    SCORE iMoveScore;
    uint64_t u64PositionSigAfterMove;
    uint64_t u64PositionSigBeforeMove;
    volatile FLAG fInUse;
}
GAME_MOVE;

typedef struct _GAME_DATA
{
    GAME_HEADER sHeader;
    DLIST_ENTRY sMoveList;
}
GAME_DATA;

extern GAME_DATA g_GameData;

POSITION *
GetRootPosition(void);

FLAG
SetRootPosition(int8_t *szFen);

void
ResetGameList(void);

uint32_t
GetMoveNumber(uint32_t uColor);

void
SetGameResultAndDescription(GAME_RESULT result);

GAME_RESULT
GetGameResult(void);

void
SetMyName(void);

void
SetOpponentsName(int8_t *sz);

void
SetMyRating(uint32_t u);

void
SetOpponentsRating(uint32_t u);

void
DumpGameList(void);

void
TellGamelistThatIPlayColor(uint32_t u);

void
DumpPgn(void);

FLAG
LoadPgn(int8_t *szPgn);

uint32_t
CountOccurrancesOfSigInOfficialGameList(uint64_t u64Sig);

FLAG
DoesSigAppearInOfficialGameList(uint64_t u64Sig);

FLAG
OfficiallyTakebackMove(void);

FLAG
OfficiallyMakeMove(MOVE mv, SCORE iMoveScore, FLAG fFast);

GAME_MOVE *
GetNthOfficialMoveRecord(uint32_t n);

void
MakeStatusLine(void);

FLAG
IsLegalDrawByRepetition(void);

//
// script.c
//
COMMAND(ScriptCommand);

COMMAND(SolutionCommand);

COMMAND(AvoidCommand);

COMMAND(IdCommand);

void
PostMoveTestSuiteReport(SEARCHER_THREAD_CONTEXT *);

FLAG
CheckTestSuiteMove(MOVE mv, SCORE iScore, uint32_t uDepth);

FLAG
WeAreRunningASuite(void);


//
// vars.c
//
COMMAND(SetCommand);

//
// root.c
//
extern uint32_t g_uSoftExtendLimit;
extern uint32_t g_uHardExtendLimit;
extern volatile MOVE_TIMER g_MoveTimer;
extern uint32_t g_uExtensionReduction[MAX_PLY_PER_SEARCH];

#ifdef PERF_COUNTERS
#define KEEP_TRACK_OF_FIRST_MOVE_FHs(x)                \
    ctx->sCounters.tree.u64BetaCutoffs++;              \
    if (x)                                             \
    {                                                  \
       ctx->sCounters.tree.u64BetaCutoffsOnFirstMove++;\
    }
#else
#define KEEP_TRACK_OF_FIRST_MOVE_FHs(x)
#endif

#define GAME_NOT_OVER 0
#define GAME_WHITE_WON 1
#define GAME_BLACK_WON 2
#define GAME_DRAW_STALEMATE 3
#define GAME_DRAW_REPETITION 4
#define GAME_ONE_LEGAL_MOVE 5
#define GAME_DRAW_FIFTY_MOVES_WO_PROGRESS 6

GAME_RESULT
Think(POSITION *pos);

GAME_RESULT
Ponder(POSITION *pos);

GAME_RESULT
Iterate(SEARCHER_THREAD_CONTEXT *ctx);

void
SetMoveTimerForSearch(FLAG fSwitchOver, uint32_t uColor);

void
SetMoveTimerToThinkForever(void);

void
ClearRootNodecountHash(void);

//
// draw.c
//
FLAG
IsDraw(SEARCHER_THREAD_CONTEXT *ctx);


//
// search.c
//
#define QPLIES_OF_NON_CAPTURE_CHECKS (1)
#define FUTILITY_BASE_MARGIN         (50) // + ctx->uPositional (min 100)
#define DO_IID
#define IID_R_FACTOR                 (TWO_PLY + HALF_PLY)

#ifndef MP
#define WE_SHOULD_STOP_SEARCHING (g_MoveTimer.bvFlags & TIMER_STOPPING)
#else
#define WE_SHOULD_STOP_SEARCHING ((g_MoveTimer.bvFlags & TIMER_STOPPING) || \
                                  (ThreadUnderTerminatingSplit(ctx)))
#endif

SCORE FASTCALL
QSearch(SEARCHER_THREAD_CONTEXT *ctx,
        SCORE iAlpha,
        SCORE iBeta);

SCORE FASTCALL
Search(SEARCHER_THREAD_CONTEXT *ctx,
       SCORE iAlpha,
       SCORE iBeta,
       uint32_t uDepth);


//
// searchsup.c
//
SCORE
ComputeMoveScore(IN SEARCHER_THREAD_CONTEXT *ctx,
                 IN MOVE mv,
                 IN uint32_t uMoveNum);

FLAG
ThreadUnderTerminatingSplit(SEARCHER_THREAD_CONTEXT *);

FLAG
WeShouldDoHistoryPruning(IN SCORE iRoughEval,
                         IN SCORE iAlpha,
                         IN SCORE iBeta,
                         IN SEARCHER_THREAD_CONTEXT *ctx,
                         IN uint32_t uRemainingDepth,
                         IN uint32_t uLegalMoves,
                         IN MOVE mv,
                         IN uint32_t uMoveNum,
                         IN int32_t iExtend);

FLAG
WeShouldTryNullmovePruning(IN SEARCHER_THREAD_CONTEXT *ctx,
                           IN SCORE iAlpha,
                           IN SCORE iBeta,
                           IN SCORE iRoughEval,
                           IN uint32_t uNullDepth);

FLAG
TryNullmovePruning(IN OUT SEARCHER_THREAD_CONTEXT *ctx,
                   IN OUT FLAG *pfThreat,
                   IN SCORE iAlpha,
                   IN SCORE iBeta,
                   IN uint32_t uNullDepth,
                   IN OUT int32_t *piOrigExtend,
                   OUT SCORE *piNullScore);

void
UpdatePV(SEARCHER_THREAD_CONTEXT *ctx, MOVE mv);

FLAG
CheckInputAndTimers(IN SEARCHER_THREAD_CONTEXT *ctx);

void
ComputeReactionToCheckExtension(IN OUT SEARCHER_THREAD_CONTEXT *ctx,
                                IN uint32_t uGenFlags,
                                IN uint32_t uMoveNum,
                                IN SCORE iRoughEval,
                                IN SCORE iAlpha,
                                IN OUT int32_t *piExtend,
                                IN OUT int32_t *piOrigExtend);

void
ComputeMoveExtension(IN OUT SEARCHER_THREAD_CONTEXT *ctx,
                     IN SCORE iAlpha,
                     IN SCORE iBeta,
                     IN uint32_t uMoveNum,
                     IN SCORE iRoughEval,
                     IN uint32_t uDepth,
                     IN OUT int32_t *piExtend);

SCORE
RescoreMovesViaSearch(IN SEARCHER_THREAD_CONTEXT *ctx,
                      IN uint32_t uDepth,
                      IN SCORE iAlpha,
                      IN SCORE iBeta);

FLAG
MateDistancePruningCutoff(IN uint32_t uPly,
                          IN FLAG fInCheck,
                          IN OUT SCORE *piBestScore,
                          IN OUT SCORE *piAlpha,
                          IN OUT SCORE *piBeta);

FLAG
CommonSearchInit(IN SEARCHER_THREAD_CONTEXT *ctx,
                 IN OUT SCORE *piAlpha,
                 IN OUT SCORE *piBeta,
                 IN OUT SCORE *piScore);

uint32_t
SelectNullmoveRFactor(IN SEARCHER_THREAD_CONTEXT *ctx,
                      IN int32_t uDepth);

#define VERIFY_BEFORE (1)
#define VERIFY_AFTER  (2)

FLAG
SanityCheckMoves(IN SEARCHER_THREAD_CONTEXT *ctx,
                 IN uint32_t uCurrent,
                 IN uint32_t uType);


//
// testsearch.c
//
FLAG
TestSearch(void);

//
// see.c
//
#define SEE_HEAPS

SCORE
SEE(POSITION *pos,
    MOVE mv);

typedef struct _SEE_THREESOME
{
    PIECE pPiece;
    COOR cLoc;
    uint32_t uVal;
} SEE_THREESOME;

typedef struct _SEE_LIST
{
    uint32_t uCount;
    SEE_THREESOME data[16];
}
SEE_LIST;

SCORE
ControlsSquareMinusPiece(uint32_t uSide,
                         POSITION *pos,
                         COOR c,
                         COOR cIgnore);

//
// testsee.c
//
SCORE
DebugSEE(POSITION *pos,
         MOVE mv);

void
TestGetAttacks(void);

//
// hash.c
//
#define NUM_HASH_ENTRIES_PER_LINE     4
#define HASH_FLAG_EXACT               0x1
#define HASH_FLAG_LOWER               0x2
#define HASH_FLAG_UPPER               0x4
#define HASH_FLAG_VALID_BOUNDS        0x7
#define HASH_FLAG_THREAT              0x8
#define HASH_FLAG_DIRTY               0xF0

typedef struct _HASH_ENTRY
{
    MOVE mv;                         // 0 1 2 3
    uint8_t uDepth;                    // 4
    uint8_t bvFlags;                   // 5 ==> d  d  d  d | thr up low exact
     int16_t iValue;             // 6 7
    uint64_t u64Sig;                   // 8 9 A B C D E F == 16 bytes
} HASH_ENTRY;

FLAG
InitializeHashSystem(void);

void
CleanupHashSystem(void);

void
ClearHashTable(void);

void
DirtyHashTable(void);

void
StoreLowerBound(MOVE mvBestMove,
           POSITION *pos,
           SCORE iValue,
           uint32_t uDepth,
           FLAG fThreat);
void
StoreExactScore(MOVE mvBestMove,
            POSITION *pos,
          SCORE iValue,
          uint32_t uDepth,
          FLAG fThreat,
          uint32_t uPly);

void
StoreUpperBound(//MOVE mvBestMove,
           POSITION *pos,
           SCORE iValue,
           uint32_t uDepth,
           FLAG fThreat);

HASH_ENTRY *
HashLookup(SEARCHER_THREAD_CONTEXT *ctx,
           uint32_t uDepth,
           uint32_t uNextDepth,
           SCORE iAlpha,
           SCORE iBeta,
           FLAG *pfThreat,
           FLAG *pfAvoidNull,
           MOVE *pHashMove,
           SCORE *piScore);

COOR
CheckHashForDangerSquare(POSITION *pos);

MOVE
GetPonderMove(POSITION *pos);

extern uint32_t g_uHashTableSizeEntries;
extern uint32_t g_uHashTableSizeBytes;
extern HASH_ENTRY *g_pHashTable;

//
// testhash.c
//
void
AnalyzeFullHashTable(void);

//
// positionhash.c
//
// IDEA: store "mate threat" flag in here?
// IDEA: store "king safety" numbers in here?
//
typedef struct _POSITION_HASH_ENTRY {
    uint64_t u64Sig;
    uint8_t cEnprise[2];
    uint8_t uEnpriseCount[2];
    uint8_t cTrapped[2];
} POSITION_HASH_ENTRY;

void
InitializePositionHashSystem(void);

void
CleanupPositionHashSystem(void);

void
StoreEnprisePiece(POSITION *pos, COOR cSquare);

void
StoreTrappedPiece(POSITION *pos, COOR cSquare);

COOR
GetEnprisePiece(POSITION *pos, uint32_t uSide);

uint32_t
GetEnpriseCount(POSITION *pos, uint32_t uSide);

COOR
GetTrappedPiece(POSITION *pos, uint32_t uSide);

FLAG
SideCanStandPat(POSITION *pos, uint32_t uSide);

uint32_t
ValueOfMaterialInTroubleDespiteMove(POSITION *pos, uint32_t uSide);

uint32_t
ValueOfMaterialInTroubleAfterNull(POSITION *pos, uint32_t uSide);

//
// pawnhash.c
//
#define COOR_TO_BIT(x)       ((x) & 0x7) + ((0x7 - ((x) >> 4)) << 3)

void
ClearPawnHashStats(void);

void
ReportPawnHashStats(void);

PAWN_HASH_ENTRY *
PawnHashLookup(SEARCHER_THREAD_CONTEXT *ctx);

//
// eval.c
//
#define LAZY_EVAL
#define LAZE_EVAL_BASE_SCORE 10
extern const int32_t g_iAhead[2];
extern const int32_t g_iBehind[2];

uint32_t 
DNABufferSizeBytes();

int8_t *
ExportEvalDNA();

FLAG
WriteEvalDNA(int8_t *szFilename);

FLAG
ImportEvalDNA(int8_t *p);

FLAG
ReadEvalDNA(int8_t *szFilename);



SCORE
Eval(SEARCHER_THREAD_CONTEXT *, SCORE, SCORE);

FLAG
EvalPasserRaces(POSITION *,
                PAWN_HASH_ENTRY *);

uint32_t
CountKingSafetyDefects(POSITION *pos,
                       uint32_t uSide);

//
// testeval.c
//
#ifdef EVAL_DUMP
void
TestEval(void);

void
EvalTraceReport(void);

void
EvalTrace(uint32_t uColor, PIECE p, COOR c, SCORE iVal, int8_t *szMessage);

SCORE
EvalSigmaForPiece(POSITION *pos, COOR c);

void
EvalTraceClear(void);

void
TestEvalWithSymmetry(void);

#define EVAL_TERM(COL, PIE, COO, VAL, ADJ, MESS) \
    (VAL) += (ADJ); \
    EvalTrace((COL), (PIE), (COO), (ADJ), (MESS));
#else
#define EVAL_TERM(COL, PIE, COO, VAL, ADJ, MESS) \
    (VAL) += (ADJ);
#endif

//
// bitboard.c
//
void
InitializeBitboards(void);

uint32_t CDECL
SlowCountBits(BITBOARD bb);

uint32_t CDECL
DeBruijnFirstBit(BITBOARD bb);

uint32_t CDECL
SlowFirstBit(BITBOARD bb);

uint32_t CDECL
SlowLastBit(BITBOARD bb);

#ifdef CROUTINES
#define CountBits SlowCountBits
#define FirstBit SlowFirstBit
#define LastBit SlowLastBit
#endif

COOR
CoorFromBitBoardRank8ToRank1(BITBOARD *pbb);

COOR
CoorFromBitBoardRank1ToRank8(BITBOARD *pbb);

//
// x86.asm
//
uint32_t CDECL
CountBits(BITBOARD bb);

uint32_t CDECL
FirstBit(BITBOARD bb);

uint32_t CDECL
LastBit(BITBOARD bb);

uint32_t CDECL
LockCompareExchange(volatile void *pDest,
                    uint32_t uExch,
                    uint32_t uComp);

uint32_t CDECL
LockIncrement(volatile void *pDest);

uint32_t CDECL
LockDecrement(volatile void *pDest);

FLAG CDECL
CanUseParallelOpcodes();

uint32_t CDECL
ParallelCompareUlong(uint32_t uComparand, void *pComparators);

uint32_t CDECL
ParallelCompareVector(void *pComparand, void *pComparators);

void CDECL
GetAttacks(SEE_LIST *pList,
           POSITION *pos,
           COOR cSquare,
           uint32_t uSide);

void CDECL
SlowGetAttacks(SEE_LIST *pList,
               POSITION *pos,
               COOR cSquare,
               uint32_t uSide);
#ifdef CROUTINES
#define GetAttacks SlowGetAttacks
#endif

#ifdef _X86_
//
// Note: this is most of the stuff that x86.asm assumes about the
// internal data structures.  If any of this fails then either the
// assembly language code needs to be updated or you need to use the C
// version of the routine in see.c instead.
//
#define ASSERT_ASM_ASSUMPTIONS \
    ASSERT(VALUE_PAWN == 100); \
    ASSERT(OFFSET_OF(uCount, SEE_LIST) == 0); \
    ASSERT(OFFSET_OF(data, SEE_LIST) == 4); \
    ASSERT(sizeof(SEE_THREESOME) == 12); \
    ASSERT(OFFSET_OF(pPiece, SEE_THREESOME) == 0); \
    ASSERT(OFFSET_OF(cLoc, SEE_THREESOME) == 4); \
    ASSERT(OFFSET_OF(uVal, SEE_THREESOME) == 8); \
    ASSERT(OFFSET_OF(cNonPawns, POSITION) == 0x478); \
    ASSERT(OFFSET_OF(uNonPawnCount, POSITION) == 0x500); \
    ASSERT(OFFSET_OF(rgSquare, POSITION) == 0); \
    ASSERT(sizeof(SQUARE) == 8); \
    ASSERT(sizeof(VECTOR_DELTA) == 4); \
    ASSERT(sizeof(g_VectorDelta) == 256 * 4); \
    ASSERT(OFFSET_OF(iVector, VECTOR_DELTA) == 0); \
    ASSERT(OFFSET_OF(iDelta, VECTOR_DELTA) == 2); \
    ASSERT(OFFSET_OF(iNegDelta, VECTOR_DELTA) == 3); \
    ASSERT(sizeof(g_PieceData) == 8 * 4 * 4); \
    ASSERT(OFFSET_OF(uValue, PIECE_DATA) == 0);
#else
#define ASSERT_ASM_ASSUMPTIONS
#endif

//
// testbitboard.c
//
void
TestBitboards(void);

//
// dynamic.c
//
extern uint32_t g_HistoryCounters[14][128];

uint32_t
GetMoveFailHighPercentage(MOVE mv);

void
UpdateDynamicMoveOrdering(SEARCHER_THREAD_CONTEXT *ctx,
                          uint32_t uRemainingDepth,
                          MOVE mvBest,
                          SCORE iScore,
                          uint32_t uCurrent);

void
NewKillerMove(SEARCHER_THREAD_CONTEXT *ctx, MOVE mv, SCORE iScore);

FLAG
InitializeDynamicMoveOrdering(void);

void
CleanupDynamicMoveOrdering(void);

void
ClearDynamicMoveOrdering(void);

void
MaintainDynamicMoveOrdering(void);

void
IncrementMoveHistoryCounter(MOVE mv, uint32_t u);

void
DecrementMoveHistoryCounter(MOVE mv, uint32_t u);

//
// split.c
//
extern volatile uint32_t g_uNumHelpersAvailable;
extern uint32_t g_uNumHelperThreads;

FLAG
InitializeParallelSearch(void);

FLAG
CleanupParallelSearch(void);

void
ClearHelperThreadIdleness(void);

void
DumpHelperIdlenessReport(void);

SCORE
StartParallelSearch(IN SEARCHER_THREAD_CONTEXT *ctx,
                    IN OUT SCORE *piAlpha,
                    IN SCORE iBeta,
                    IN OUT SCORE *piBestScore,
                    IN OUT MOVE *pmvBest,
                    IN uint32_t uMoveNum,
                    IN int32_t iPositionExtend,
                    IN uint32_t uDepth);

void
InitializeSearcherContext(POSITION *pos, SEARCHER_THREAD_CONTEXT *ctx);

void
ReInitializeSearcherContext(POSITION *pos, SEARCHER_THREAD_CONTEXT *ctx);

void
InitializeLightweightSearcherContext(POSITION *pos,
                                     LIGHTWEIGHT_SEARCHER_CONTEXT *ctx);

//
// book.c
//

#define BOOK_PROBE_MISS_LIMIT  (7)
#define FLAG_DISABLED          (1)
#define FLAG_ALWAYSPLAY        (2)
#define FLAG_DELETED           (4)

#define BOOKMOVE_SELECT_MOVE   (1)
#define BOOKMOVE_DUMP          (2)

#pragma pack(4)
typedef struct _BOOK_ENTRY
{
    uint64_t u64Sig;                            // 8 bytes
    uint64_t u64NextSig;                        // 8 bytes
    MOVE mvNext;                              // 4 bytes
    uint32_t uWins;                              // 4 bytes
    uint32_t uDraws;                             // 4 bytes
    uint32_t uLosses;                            // 4 bytes
    BITV bvFlags;                             // 4 bytes
}
BOOK_ENTRY;
#pragma pack()

typedef struct _OPENING_NAME_MAPPING
{
    uint64_t u64Sig;
    int8_t *szString;
}
OPENING_NAME_MAPPING;

#define BOOK_EDITING_RECORD       "bkedit.edt"
#define OPENING_LEARNING_FILENAME "bklearn.bin"

typedef struct _OPENING_LEARNING_ENTRY
{
    uint64_t u64Sig;
    uint32_t uWhiteWins;
    uint32_t uDraws;
    uint32_t uBlackWins;
}
OPENING_LEARNING_ENTRY;

FLAG
InitializeOpeningBook(void);

void
ResetOpeningBook(void);

void
CleanupOpeningBook(void);

MOVE
BookMove(POSITION *pos,
         BITV bvFlags);

COMMAND(BookCommand);

extern uint32_t g_uBookProbeFailures;
extern FLAG g_fTournamentMode;
extern int8_t *g_szBookName;

//
// bench.c
//
COMMAND(BenchCommand);

//
// testdraw.c
//
void
TestDraw(void);

//
// probe.c
//
FLAG
ProbeEGTB(SEARCHER_THREAD_CONTEXT *ctx, SCORE *score);

void
InitializeEGTB(void);

void
CleanupEGTB(void);

//
// dumptree.c
//
#ifdef DUMP_TREE

void
InitializeTreeDump(void);

void
CleanupTreeDump(void);

void
DTEnterNode(SEARCHER_THREAD_CONTEXT *ctx,
            uint32_t uDepth,
            FLAG fIsQNode,
            SCORE iAlpha,
            SCORE iBeta);

void
DTLeaveNode(SEARCHER_THREAD_CONTEXT *ctx,
            FLAG fQNode,
            SCORE iBestScore,
            MOVE mvBestMove);

void CDECL
DTTrace(uint32_t uPly, int8_t *szMessage, ...);

#define DTTRACE(...) DTTrace(ctx->uPly, __VA_ARGS__)

#else

#define DTTRACE(...)

#define DTEnterNode(...)

#define DTLeaveNode(...)

#define InitializeTreeDump(...)

#define CleanupTreeDump(...)

#endif

//
// testsup.c
//
FLAG
IsBoardLegal(POSITION *pos);

void
GenerateRandomLegalPosition(POSITION *pos);

void
GenerateRandomLegalSymetricPosition(POSITION *pos);

//
// recogn.c
//
#define UNRECOGNIZED (0)
#define RECOGN_EXACT (1)
#define RECOGN_UPPER (2)
#define RECOGN_LOWER (3)
#define RECOGN_EGTB  (4)

void
InitializeInteriorNodeRecognizers(void);

uint32_t
RecognLookup(SEARCHER_THREAD_CONTEXT *ctx,
             SCORE *piScore,
             FLAG fProbeEGTB);

#ifdef EVAL_HASH
void
ClearEvalHashStats(void);

void
ReportEvalHashStats(void);

SCORE
ProbeEvalHash(SEARCHER_THREAD_CONTEXT *ctx);

SCORE
GetRoughEvalScore(IN SEARCHER_THREAD_CONTEXT *ctx,
                  IN SCORE iAlpha,
                  IN SCORE iBeta,
                  IN FLAG fUseHash);

void
StoreEvalHash(SEARCHER_THREAD_CONTEXT *ctx, SCORE iScore);
#endif // EVAL_HASH

#endif // CHESS
